@@ -1,4 +1,5 @@
 // Copyright (c) 2020 Red Hat, Inc.
+// Copyright Contributors to the Open Cluster Management project
 
 package controllers
 
@@ -9,12 +10,14 @@ import (
 	"math/big"
 	"os"
 	"reflect"
+	"strconv"
 	"time"
 
+	"k8s.io/apimachinery/pkg/labels"
 	"k8s.io/apimachinery/pkg/util/intstr"
 
 	"github.com/go-logr/logr"
-	searchv1alpha1 "github.com/stolostron/search-operator/api/v1alpha1"
+	searchv1alpha1 "github.com/open-cluster-management/search-operator/api/v1alpha1"
 	appv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
@@ -53,6 +56,8 @@ const (
 	statusNoPersistence       = "Redisgraph pod running with persistence disabled"
 	redisUser                 = int64(10001)
 	defaultPvcName            = "search-redisgraph-pvc-0"
+	statusUpdateError         = "Error updating operator/customization status. "
+	errorLogStr               = "Error: "
 )
 
 var (
@@ -65,8 +70,12 @@ var (
 	storageSize          = "10Gi"
 	namespace            = os.Getenv("WATCH_NAMESPACE")
 	releaseName          = os.Getenv("RELEASE_NAME")
+	//Keeping these here as the pod will restart everytime when ENV is updated and we will read the updated values
+	deployRedisgraphPod, deployVarPresent = os.LookupEnv("DEPLOY_REDISGRAPH")
+	deploy, deployVarErr                  = strconv.ParseBool(deployRedisgraphPod)
 )
 var startingSpec searchv1alpha1.SearchCustomizationSpec
+var deployStatus bool
 
 func (r *SearchOperatorReconciler) Reconcile(req ctrl.Request) (ctrl.Result, error) {
 	_ = context.Background()
@@ -137,14 +146,40 @@ func (r *SearchOperatorReconciler) Reconcile(req ctrl.Request) (ctrl.Result, err
 	err = r.setupSecret(r.Client, instance)
 	if err != nil {
 		// Error setting up secret - requeue the request.
+		if err = updateCRs(r.Client, instance, redisNotRunning,
+			custom, persistence, storageClass, storageSize, customValuesInuse); err != nil {
+			r.Log.Info(statusUpdateError, errorLogStr, err)
+		}
 		return ctrl.Result{}, err
 	}
 
 	//Read the searchoperator status
 	persistenceStatus := instance.Status.PersistenceStatus
-
+	if instance.Status.DeployRedisgraph != nil {
+		deployStatus = *instance.Status.DeployRedisgraph
+	}
 	// Setup RedisGraph Deployment
 	r.Log.Info(fmt.Sprintf("Config in  Use Persistence/AllowDegrade %t/%t", persistence, allowdegrade))
+	//if deploy env variable is false, don't deploy Redisgraph pod
+	if deployVarPresent && deployVarErr == nil && !deploy {
+		err := deleteRedisStatefulSet(r.Client) //if redisgraph pod is already deployed, delete it.
+		if err != nil {
+			if err = updateCRs(r.Client, instance, redisNotRunning,
+				custom, persistence, storageClass, storageSize, customValuesInuse); err != nil {
+				r.Log.Info(statusUpdateError, errorLogStr, err)
+			}
+			return ctrl.Result{}, err
+		}
+		r.Log.Info(`Not deploying the database. This is not an error, it's a current limitation in this environment.
+	The search feature is not operational.  More info: https://github.com/open-cluster-management/community/issues/34`)
+		//Write Status
+		err = updateCRs(r.Client, instance, redisNotRunning,
+			custom, persistence, storageClass, storageSize, customValuesInuse)
+		if err != nil {
+			return ctrl.Result{}, err
+		}
+		return ctrl.Result{}, nil
+	}
 	if persistence {
 		expectedSts := r.expectedStatefulSet(r.Client,
 			instance, true, persistence)
@@ -152,7 +187,7 @@ func (r *SearchOperatorReconciler) Reconcile(req ctrl.Request) (ctrl.Result, err
 		// Do nothing if status (persistence and deploy) in searchoperator is up-to-date with statusUsingPVC
 		// and statefulset is available and up-to-date
 		// and pod is running with PVC volume
-		if persistenceStatus == statusUsingPVC && isStatefulSetAvailable(r.Client) &&
+		if persistenceStatus == statusUsingPVC && deployStatus == deploy && isStatefulSetAvailable(r.Client) &&
 			!statefulSetNeedsUpdate(r.Client, expectedSts) && r.isPodRunning(true, 1) {
 			r.Log.Info("Redisgraph Pod running successfully with PVC.")
 			return ctrl.Result{}, nil
@@ -163,13 +198,29 @@ func (r *SearchOperatorReconciler) Reconcile(req ctrl.Request) (ctrl.Result, err
 		// Do nothing if status (persistence and deploy) in searchoperator is up-to-date with statusDegradedEmptyDir
 		// and statefulset is available and up-to-date
 		// and pod is running with emptyDir volume
-		if allowdegrade && persistenceStatus == statusDegradedEmptyDir && isStatefulSetAvailable(r.Client) &&
+		if allowdegrade && persistenceStatus == statusDegradedEmptyDir &&
+			deployStatus == deploy && isStatefulSetAvailable(r.Client) &&
 			!statefulSetNeedsUpdate(r.Client, expectedSts) && r.isPodRunning(false, 1) {
 			r.Log.Info("Redisgraph Pod running successfully with EmptyDir.")
 			return ctrl.Result{}, nil
 		}
+		//Restart search-collector pod while setting up Redisgraph pod
+		if deployVarPresent && deployVarErr == nil && deploy {
+			srchOp, err := fetchSrchOperator(r.Client, instance)
+			//If Redisgraph was disabled, collector will be in a 10 minute timeout loop.
+			//Restart collector and api pods while deploying Redisgraph.
+			if err == nil && srchOp.Status.DeployRedisgraph != nil && *srchOp.Status.DeployRedisgraph == false {
+				r.Log.Info("Restarting search-collector and search-api pods")
+				//restart collector and api pods
+				r.restartSearchComponents()
+			}
+		}
 		pvcError := setupVolume(r.Client)
 		if pvcError != nil {
+			if err = updateCRs(r.Client, instance, redisNotRunning,
+				custom, persistence, storageClass, storageSize, customValuesInuse); err != nil {
+				r.Log.Info(statusUpdateError, errorLogStr, err)
+			}
 			return ctrl.Result{}, pvcError
 		}
 		r.Log.Info("PVC volume set up successfully")
@@ -188,11 +239,19 @@ func (r *SearchOperatorReconciler) Reconcile(req ctrl.Request) (ctrl.Result, err
 			r.Log.Info("Degrading Redisgraph deployment to use empty dir.")
 			err := deleteRedisStatefulSet(r.Client)
 			if err != nil {
+				if err = updateCRs(r.Client, instance, redisNotRunning,
+					custom, persistence, storageClass, storageSize, customValuesInuse); err != nil {
+					r.Log.Info(statusUpdateError, errorLogStr, err)
+				}
 				return ctrl.Result{}, err
 			}
 			r.Log.Info("Deleted statefulset to move to emptyDir")
 			err = deletePVC(r.Client)
 			if err != nil {
+				if err = updateCRs(r.Client, instance, redisNotRunning,
+					custom, persistence, storageClass, storageSize, customValuesInuse); err != nil {
+					r.Log.Info(statusUpdateError, errorLogStr, err)
+				}
 				return ctrl.Result{}, err
 			}
 			r.Log.Info("Deleted PVC to move to emptyDir")
@@ -222,7 +281,7 @@ func (r *SearchOperatorReconciler) Reconcile(req ctrl.Request) (ctrl.Result, err
 		}
 	} else {
 		if isStatefulSetAvailable(r.Client) && r.isPodRunning(false, 1) &&
-			persistenceStatus == statusNoPersistence {
+			persistenceStatus == statusNoPersistence && deployStatus == deploy {
 			return ctrl.Result{}, nil
 		}
 		r.Log.Info("Using Deployment with persistence disabled")
@@ -250,10 +309,48 @@ func (r *SearchOperatorReconciler) reconcileOnError(instance *searchv1alpha1.Sea
 	var err error
 	if err = updateCRs(r.Client, instance, status, custom, false,
 		storageClass, storageSize, customValuesInuse); err != nil {
-		r.Log.Info("Error updating operator/customization status. ", "Error: ", err)
+		r.Log.Info(statusUpdateError, errorLogStr, err)
 	}
 	if err = deleteRedisStatefulSet(r.Client); err != nil {
-		r.Log.Info("Error deleting statefulset. ", "Error: ", err)
+		r.Log.Info("Error deleting statefulset. ", errorLogStr, err)
+	}
+}
+
+// Restart search collector and api pods
+func (r *SearchOperatorReconciler) restartSearchComponents() {
+	allComponents := map[string]map[string]string{}
+	allComponents["Search-collector"] = map[string]string{"app": "search-prod", "component": "search-collector"}
+	allComponents["Search-api"] = map[string]string{"app": "search", "component": "search-api"}
+
+	for compName, compLabels := range allComponents {
+		opts := getOptions(compLabels)
+		podList := &corev1.PodList{}
+		err := r.Client.List(context.TODO(), podList, opts...)
+		if err != nil || len(podList.Items) == 0 {
+			if len(podList.Items) == 0 {
+				r.Log.Info(fmt.Sprintf("Failed to find %s pods. %d pods found", compName, len(podList.Items)))
+			}
+			r.Log.Info(fmt.Sprintf("Error listing pods for component: %s", compName), "Err:", err)
+			continue
+		}
+
+		for _, item := range podList.Items {
+			compPod := &corev1.Pod{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      item.Name,
+					Namespace: item.Namespace,
+				},
+			}
+			err := r.Client.Delete(context.TODO(), compPod)
+			if err != nil && !errors.IsNotFound(err) {
+				r.Log.Info("Failed to delete pods for ", "component", compName)
+				r.Log.Info(err.Error())
+				// Not needed to act on the error as restarting is to offset the timeout
+				// search will continue to function
+				continue
+			}
+			r.Log.Info(fmt.Sprintf("%s pod deleted. Namespace/Name: %s/%s", compName, item.Namespace, item.Name))
+		}
 	}
 }
 
@@ -314,12 +411,12 @@ func compareLabels(metadataLabels, ssetLabels map[string]string) bool {
 			continue
 		} else {
 			allLabelsPresent = false
+			log.Info("Not all labels present in statefulset. Label: ", label, "not found in metadata")
 			break
 		}
 	}
 	return allLabelsPresent
 }
-
 func (r *SearchOperatorReconciler) getStatefulSet(cr *searchv1alpha1.SearchOperator,
 	rdbVolumeSource corev1.VolumeSource, saverdb string) *appv1.StatefulSet {
 	sset := &appv1.StatefulSet{}
@@ -453,7 +550,7 @@ func (r *SearchOperatorReconciler) getStatefulSet(cr *searchv1alpha1.SearchOpera
 			Name: "redis-graph-certs",
 			VolumeSource: corev1.VolumeSource{
 				Secret: &corev1.SecretVolumeSource{
-					SecretName: "search-redisgraph-secrets",
+					SecretName: "search-redisgraph-certs",
 					Items: []corev1.KeyToPath{
 						{
 							Key:  "tls.crt",
@@ -497,7 +594,6 @@ func (r *SearchOperatorReconciler) getStatefulSet(cr *searchv1alpha1.SearchOpera
 	}
 	return sset
 }
-
 func updateCRs(kclient client.Client, operatorCR *searchv1alpha1.SearchOperator, status string,
 	customizationCR *searchv1alpha1.SearchCustomization, persistence bool, storageClass string,
 	storageSize string, customValuesInuse bool) error {
@@ -516,20 +612,34 @@ func updateCRs(kclient client.Client, operatorCR *searchv1alpha1.SearchOperator,
 	return nil
 }
 
-func updateOperatorCR(kclient client.Client, cr *searchv1alpha1.SearchOperator, status string) error {
+func fetchSrchOperator(kclient client.Client, cr *searchv1alpha1.SearchOperator) (
+	*searchv1alpha1.SearchOperator, error) {
 	found := &searchv1alpha1.SearchOperator{}
 	err := kclient.Get(context.TODO(), types.NamespacedName{Name: cr.Name, Namespace: cr.Namespace}, found)
+	return found, err
+}
+func fetchSrchCustomization(kclient client.Client, cr *searchv1alpha1.SearchCustomization) (
+	*searchv1alpha1.SearchCustomization, error) {
+	found := &searchv1alpha1.SearchCustomization{}
+	err := kclient.Get(context.TODO(), types.NamespacedName{Name: cr.Name, Namespace: cr.Namespace}, found)
+	return found, err
+}
+func updateOperatorCR(kclient client.Client, cr *searchv1alpha1.SearchOperator, status string) error {
+	cr, err := fetchSrchOperator(kclient, cr)
 	if err != nil {
 		log.Error(err, fmt.Sprintf("Failed to get SearchOperator %s/%s ", cr.Namespace, cr.Name))
 		return err
 	}
 	cr.Status.PersistenceStatus = status
+	if deployVarPresent && deployVarErr == nil {
+		cr.Status.DeployRedisgraph = &deploy
+	}
 	err = kclient.Status().Update(context.TODO(), cr)
 	if err != nil {
 		if errors.IsConflict(err) {
 			log.Info("Failed to update status Object has been modified")
 		}
-		log.Error(err, fmt.Sprintf("Failed to update %s/%s status ", cr.Namespace, cr.Name))
+		log.Info(fmt.Sprintf("Failed to update %s/%s status. Error: %s", cr.Namespace, cr.Name, err.Error()))
 		return err
 	} else {
 		log.Info(fmt.Sprintf("Updated CR status with persistence %s  ", cr.Status.PersistenceStatus))
@@ -539,8 +649,7 @@ func updateOperatorCR(kclient client.Client, cr *searchv1alpha1.SearchOperator,
 
 func updateCustomizationCR(kclient client.Client, cr *searchv1alpha1.SearchCustomization,
 	persistence bool, storageClass string, storageSize string) error {
-	found := &searchv1alpha1.SearchCustomization{}
-	err := kclient.Get(context.TODO(), types.NamespacedName{Name: cr.Name, Namespace: cr.Namespace}, found)
+	cr, err := fetchSrchCustomization(kclient, cr)
 	if err != nil {
 		log.Error(err, fmt.Sprintf("Failed to get SearchCustomization %s/%s ", cr.Namespace, cr.Name))
 		return err
@@ -553,7 +662,7 @@ func updateCustomizationCR(kclient client.Client, cr *searchv1alpha1.SearchCusto
 		if errors.IsConflict(err) {
 			log.Info("Failed to update status Object has been modified")
 		}
-		log.Error(err, fmt.Sprintf("Failed to update %s/%s status ", cr.Namespace, cr.Name))
+		log.Info(fmt.Sprintf("Failed to update %s/%s status. Error:  %s", cr.Namespace, cr.Name, err.Error()))
 		return err
 	} else {
 		log.Info(fmt.Sprintf("Updated CR status with custom persistence %t ", cr.Status.Persistence))
@@ -577,7 +686,6 @@ func statefulSetNeedsUpdate(client client.Client, deployment *appv1.StatefulSet)
 		}
 	}
 }
-
 func updateRedisStatefulSet(client client.Client, deployment *appv1.StatefulSet) {
 	found := &appv1.StatefulSet{}
 	err := client.Get(context.TODO(), types.NamespacedName{Name: statefulSetName, Namespace: namespace}, found)
@@ -604,8 +712,8 @@ func updateRedisStatefulSet(client client.Client, deployment *appv1.StatefulSet)
 		log.Info("Volume source and/or metadata updated for redisgraph Statefulset")
 		return
 	}
-}
 
+}
 func deleteRedisStatefulSet(client client.Client) error {
 	statefulset := &appv1.StatefulSet{
 		ObjectMeta: metav1.ObjectMeta{
@@ -866,3 +974,14 @@ func (r *SearchOperatorReconciler) setupSecret(client client.Client, cr *searchv
 	}
 	return nil
 }
+
+func getOptions(opts map[string]string) []client.ListOption {
+	listOptions := []client.ListOption{}
+	listOption := client.ListOptions{
+		LabelSelector: labels.SelectorFromSet(opts),
+		Namespace:     namespace,
+		Limit:         2, //setting this to 2 as a safety net while deleting pods
+	}
+	listOptions = append(listOptions, &listOption)
+	return listOptions
+}
