@@ -1,12 +1,15 @@
 // Copyright (c) 2020 Red Hat, Inc.
+// Copyright Contributors to the Open Cluster Management project
 
 package controllers
 
 import (
 	"context"
+	"os"
+	"strconv"
 	"testing"
 
-	searchv1alpha1 "github.com/stolostron/search-operator/api/v1alpha1"
+	searchv1alpha1 "github.com/open-cluster-management/search-operator/api/v1alpha1"
 	"github.com/stretchr/testify/assert"
 	appv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
@@ -317,6 +320,39 @@ func Test_UnschedulablePodWithOutPersistence(t *testing.T) {
 	assert.Equal(t, statusFailedNoPersistence, instance.Status.PersistenceStatus, "Search Operator status updated with statusFailedNoPersistence as expected.")
 }
 
+func Test_DoNotDeployRedisPod(t *testing.T) {
+	testSetup := commonSetup()
+	//Set DEPLOY_REDISGRAPH env to false to stop redisgraph pod from being deployed
+	os.Setenv("DEPLOY_REDISGRAPH", "false")
+	deployRedisgraphPod, deployVarPresent = os.LookupEnv("DEPLOY_REDISGRAPH")
+	deploy, deployVarErr = strconv.ParseBool(deployRedisgraphPod)
+	req := testSetup.request
+	testStatefulset := testSetup.statefulsetWithPVC
+
+	client := fake.NewFakeClientWithScheme(testSetup.scheme, testSetup.srchOperator, testSetup.secret, testSetup.pvc, testSetup.statefulsetWithPVC)
+
+	nilSearchOperator := SearchOperatorReconciler{client, log, testSetup.scheme}
+	var err error
+	instance := &searchv1alpha1.SearchOperator{}
+	err = client.Get(context.TODO(), req.NamespacedName, instance)
+	assert.Nil(t, err, "Expected search Operator to be created. Got error: %v", err)
+
+	_, err = nilSearchOperator.Reconcile(req)
+	assert.Nil(t, err, "Expected search Operator reconcile to complete successfully. Got error: %v", err)
+
+	err = client.Get(context.TODO(), req.NamespacedName, instance)
+	assert.Nil(t, err, "Expected search Operator to be created. Got error: %v", err)
+
+	foundStatefulset := &appv1.StatefulSet{}
+	err = client.Get(context.TODO(), types.NamespacedName{Name: testStatefulset.Name, Namespace: testStatefulset.Namespace}, foundStatefulset)
+	assert.True(t, errors.IsNotFound(err), "Expected error: redisgraph statefulset to be Not Found. Got %v", err.Error())
+	assert.Equal(t, redisNotRunning, instance.Status.PersistenceStatus, "Search Operator status not set as expected.")
+	//Resetting the variables
+	os.Unsetenv("DEPLOY_REDISGRAPH")
+	deployRedisgraphPod, deployVarPresent = os.LookupEnv("DEPLOY_REDISGRAPH")
+	deploy, deployVarErr = strconv.ParseBool(deployRedisgraphPod)
+}
+
 func Test_UnschedulablePodWithDisAllowDegradedMode(t *testing.T) {
 	testSetup := commonSetup()
 
@@ -370,6 +406,29 @@ func TestGetPVC(t *testing.T) {
 	assert.Nil(t, pvc.Spec.StorageClassName, "Expected empty StorageClassName. Got: %s", pvc.Spec.StorageClassName)
 }
 
+func TestRestartCollector(t *testing.T) {
+	//create fake Collector Pod
+	labels := map[string]string{}
+	labels["app"] = "search-prod"
+	labels["component"] = "search-collector"
+	testSetup := commonSetup()
+	collectorPod := &corev1.Pod{ObjectMeta: metav1.ObjectMeta{Namespace: namespace, Name: "search-collector-pod", Labels: labels}}
+
+	client := fake.NewFakeClientWithScheme(testSetup.scheme, testSetup.srchOperator, collectorPod)
+	nilSearchOperator := SearchOperatorReconciler{client, log, testSetup.scheme}
+
+	nilSearchOperator.restartSearchComponents()
+	req := reconcile.Request{
+		NamespacedName: types.NamespacedName{
+			Namespace: namespace,
+			Name:      "search-collector-pod",
+		},
+	}
+	collectorPod1 := &corev1.Pod{}
+	err := client.Get(context.TODO(), req.NamespacedName, collectorPod1)
+	assert.True(t, errors.IsNotFound(err), "Expected error: SearchCollector pod to be Not Found. Got %v", err.Error())
+}
+
 func createFakeNamedPVC(requestBytes string, namespace string, userAnnotations map[string]string) *corev1.PersistentVolumeClaim {
 	annotations := map[string]string{}
 	for k, v := range userAnnotations {
@@ -398,7 +457,7 @@ func createFakeRedisGraphPod(namespace string, persistence, schedulable bool) *c
 	labels := map[string]string{}
 	labels["app"] = appName
 	labels["component"] = component
-	image := "quay.io/stolostron/search-operator:latest"
+	image := "quay.io/open-cluster-management/search-operator:latest"
 	containerStatuses := []corev1.ContainerStatus{}
 	containerStatus := corev1.ContainerStatus{Ready: true}
 	containerStatuses = append(containerStatuses, containerStatus)
